@@ -198,6 +198,7 @@
 extern "C" {
 #endif
 
+#include <rte_compat.h>
 #include <rte_common.h>
 #include <rte_dev.h>
 #include <rte_mbuf.h>
@@ -205,21 +206,22 @@ extern "C" {
 #define RTE_REGEXDEV_NAME_MAX_LEN RTE_DEV_NAME_MAX_LEN
 
 extern int rte_regexdev_logtype;
+#define RTE_LOGTYPE_REGEXDEV rte_regexdev_logtype
 
-#define RTE_REGEXDEV_LOG(level, ...) \
-	rte_log(RTE_LOG_ ## level, rte_regexdev_logtype, "" __VA_ARGS__)
+#define RTE_REGEXDEV_LOG_LINE(level, ...) \
+	RTE_LOG_LINE(level, REGEXDEV, "" __VA_ARGS__)
 
 /* Macros to check for valid port */
 #define RTE_REGEXDEV_VALID_DEV_ID_OR_ERR_RET(dev_id, retval) do { \
 	if (!rte_regexdev_is_valid_dev(dev_id)) { \
-		RTE_REGEXDEV_LOG(ERR, "Invalid dev_id=%u\n", dev_id); \
+		RTE_REGEXDEV_LOG_LINE(ERR, "Invalid dev_id=%u", dev_id); \
 		return retval; \
 	} \
 } while (0)
 
 #define RTE_REGEXDEV_VALID_DEV_ID_OR_RET(dev_id) do { \
 	if (!rte_regexdev_is_valid_dev(dev_id)) { \
-		RTE_REGEXDEV_LOG(ERR, "Invalid dev_id=%u\n", dev_id); \
+		RTE_REGEXDEV_LOG_LINE(ERR, "Invalid dev_id=%u", dev_id); \
 		return; \
 	} \
 } while (0)
@@ -612,6 +614,8 @@ struct rte_regexdev_info {
 	/**< Maximum payload size for a pattern match request or scan.
 	 * @see RTE_REGEXDEV_CFG_CROSS_BUFFER_SCAN_F
 	 */
+	uint16_t max_segs;
+	/**< Maximum number of mbuf segments that can be chained together. */
 	uint32_t max_rules_per_group;
 	/**< Maximum rules supported per group by this device. */
 	uint16_t max_groups;
@@ -1243,7 +1247,6 @@ rte_regexdev_dump(uint8_t dev_id, FILE *f);
  * @see struct rte_regex_ops::matches
  */
 struct rte_regexdev_match {
-	RTE_STD_C11
 	union {
 		uint64_t u64;
 		struct {
@@ -1257,7 +1260,6 @@ struct rte_regexdev_match {
 			 */
 			uint16_t start_offset;
 			/**< Starting Byte Position for matched rule. */
-			RTE_STD_C11
 			union {
 				uint16_t len;
 				/**< Length of match in bytes */
@@ -1390,7 +1392,6 @@ struct rte_regex_ops {
 	 */
 
 	/* W3 */
-	RTE_STD_C11
 	union {
 		uint64_t user_id;
 		/**< Application specific opaque value. An application may use
@@ -1403,7 +1404,6 @@ struct rte_regex_ops {
 	};
 
 	/* W4 */
-	RTE_STD_C11
 	union {
 		uint64_t cross_buf_id;
 		/**< ID used by the RegEx device in order to support cross
@@ -1473,9 +1473,10 @@ rte_regexdev_enqueue_burst(uint8_t dev_id, uint16_t qp_id,
 	struct rte_regexdev *dev = &rte_regex_devices[dev_id];
 #ifdef RTE_LIBRTE_REGEXDEV_DEBUG
 	RTE_REGEXDEV_VALID_DEV_ID_OR_ERR_RET(dev_id, -EINVAL);
-	RTE_FUNC_PTR_OR_ERR_RET(*dev->enqueue, -ENOTSUP);
+	if (*dev->enqueue == NULL)
+		return -ENOTSUP;
 	if (qp_id >= dev->data->dev_conf.nb_queue_pairs) {
-		RTE_REGEXDEV_LOG(ERR, "Invalid queue %d\n", qp_id);
+		RTE_REGEXDEV_LOG_LINE(ERR, "Invalid queue %d", qp_id);
 		return -EINVAL;
 	}
 #endif
@@ -1532,9 +1533,10 @@ rte_regexdev_dequeue_burst(uint8_t dev_id, uint16_t qp_id,
 	struct rte_regexdev *dev = &rte_regex_devices[dev_id];
 #ifdef RTE_LIBRTE_REGEXDEV_DEBUG
 	RTE_REGEXDEV_VALID_DEV_ID_OR_ERR_RET(dev_id, -EINVAL);
-	RTE_FUNC_PTR_OR_ERR_RET(*dev->dequeue, -ENOTSUP);
+	if (*dev->dequeue == NULL)
+		return -ENOTSUP;
 	if (qp_id >= dev->data->dev_conf.nb_queue_pairs) {
-		RTE_REGEXDEV_LOG(ERR, "Invalid queue %d\n", qp_id);
+		RTE_REGEXDEV_LOG_LINE(ERR, "Invalid queue %d", qp_id);
 		return -EINVAL;
 	}
 #endif
